# Copyright (c) 2017 Presslabs SRL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from django.test import Client
from django.urls import reverse
from freezegun import freeze_time

from payu.models import PayUIPN


@pytest.mark.django_db
@pytest.mark.parametrize('method, status_code', [
    ('get', 200),
    ("post", 200),
    ('put', 405),
    ('patch', 405),
    ('delete', 405)
])
def test_ipn_view_methods_access(method, status_code):
    client = Client()
    assert getattr(client, method)(reverse('payu-ipn')).status_code == status_code


@pytest.mark.django_db
@freeze_time("2022-01-05 11:49:52")
def test_ipn_view_valid_payload():
    post_data = {
        'HASH': '30dcbb067e28aee6cfefbf45e9285d7b',
        'REFNO': '10',
        'REFNOEXT': '1',
        'ORDERNO': '1',
        'ORDERSTATUS': 'TEST',
        'PAYMETHOD_CODE': 'CCVMC',
    }
    client = Client()

    response = client.post(reverse('payu-ipn'), post_data)
    assert response.content == b'<EPAYMENT>20220105114952|2988b8b159454b1ed380abe25e557c9e</EPAYMENT>'

    ipn = PayUIPN.objects.filter(REFNO=post_data['REFNO']).first()
    assert ipn

    for field in post_data:
        assert getattr(ipn, field) == post_data[field]

    assert ipn.flag_info == ''
    assert not ipn.flag



# Copyright (c) 2017 Presslabs SRL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

import django
from django.conf import settings


settings.configure(
    DEBUG=True,
    ROOT_URLCONF='payu.urls',
    DATABASES={
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
        }
    },
    PAYU_MERCHANT='PAYUDEMO',
    PAYU_KEY='1231234567890123',
    INSTALLED_APPS=('django.contrib.auth',
                    'django.contrib.contenttypes',
                    'django.contrib.sessions',
                    'django.contrib.admin',
                    'payu',),
    SECRET_KEY='dummy'
)


django.setup()



#
# Copyright 2012-2016 PressLabs SRL
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
import hashlib
import hmac
from datetime import datetime
from collections import OrderedDict

import pytz
import requests

from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver

from payu.signals import (payment_completed, payment_authorized,
                          payment_flagged, alu_token_created)
from payu.conf import (PAYU_PAYMENT_STATUS, PAYU_IDN_URL,
                       PAYU_MERCHANT, PAYU_MERCHANT_KEY)


class PayUIPN(models.Model):
    REFNO = models.CharField(max_length=9, verbose_name='ePayment reference')
    REFNOEXT = models.CharField(max_length=100, verbose_name='Merchant reference')
    ORDERNO = models.CharField(max_length=6, verbose_name='Merchant order #')
    ORDERSTATUS = models.CharField(max_length=18, choices=PAYU_PAYMENT_STATUS,
                                   verbose_name='Status')
    HASH = models.CharField(max_length=64)
    PAYMETHOD_CODE = models.CharField(max_length=10,
                                      verbose_name='Payment method code')

    SALEDATE = models.DateTimeField(blank=True, null=True,
                                    verbose_name='Sale date')
    COMPLETE_DATE = models.DateTimeField(blank=True, null=True,
                                         verbose_name='Complete date')
    PAYMENTDATE = models.DateTimeField(blank=True, null=True,
                                       verbose_name='Payment date')

    PAYMETHOD = models.CharField(blank=True, null=True, max_length=100,
                                 verbose_name='Payment method')
    FIRSTNAME = models.CharField(blank=True, null=True, max_length=40,
                                 help_text='Client\'s first name')
    LASTNAME = models.CharField(blank=True, null=True, max_length=40,
                                help_text='Client\'s last name')
    IDENTITY_NO = models.CharField(blank=True, null=True, max_length=15,
                                   help_text='Customer ID Card series and \
                                   number (Series / Number - available \
                                   only for Romanian customers)')
    IDENTITY_ISSUER = models.CharField(blank=True, null=True, max_length=100,
                                       help_text='IDENTITY_NO ID Card \
                                       issuer authority ')
    CARD_TYPE = models.CharField(blank=True, null=True, max_length=100,
                                 help_text='Used credit card type. \
                                 Ex: "Visa" or "MasterCard"')
    IDENTITY_CNP = models.CharField(blank=True, null=True, max_length=13,
                                    help_text='Customer\'s personal numeric \
                                    code, available only for Romanian customers.')
    COMPANY = models.CharField(blank=True, null=True, max_length=40,
                               help_text='Company name (maximum length: \
                                             40 characters) ')
    REGISTRATIONNUMBER = models.CharField(blank=True, null=True, max_length=40,
                                          help_text='Company\'s Commerce \
                                          Registry registration number \
                                          (maximum length: 40 characters)')
    FISCALCODE = models.CharField(blank=True, null=True, max_length=40,
                                  help_text='Unique Registration Number / \
                                  VAT ID (maximum length: 40 characters)')
    CBANKNAME = models.CharField(blank=True, null=True, max_length=40,
                                 help_text='Company\'s Bank (maximum \
                                 length: 40 characters) ')
    CBANKACCOUNT = models.CharField(blank=True, null=True, max_length=50,
                                    help_text='Company\'s Bank Account \
                                    (maximum length: 50 characters)')
    ADDRESS1 = models.CharField(blank=True, null=True, max_length=100,
                                help_text='Address (maximum length: 100 \
                                characters)')
    ADDRESS2 = models.CharField(blank=True, null=True, max_length=100,
                                help_text='Additional Address info \
                                (maximum length: 100 characters)')
    CITY = models.CharField(blank=True, null=True, max_length=30,
                            help_text='City (maximum length: 30 characters)')
    STATE = models.CharField(blank=True, null=True, max_length=30,
                             help_text='State/Sector/County (maximum \
                             length: 30 characters)')
    ZIPCODE = models.CharField(blank=True, null=True, max_length=20,
                               help_text='ZIP/Postal Code (maximum length: \
                               20 characters)')
    COUNTRY = models.CharField(blank=True, null=True, max_length=50,
                               help_text='Country (maximum length: 50 \
                               characters)')
    COUNTRY_CODE = models.CharField(blank=True, null=True, max_length=10,
                                    help_text='Country (maximum length: 10 \
                                    characters)')
    PHONE = models.CharField(blank=True, null=True, max_length=40,
                             help_text='Phone number (maximum length: 40 \
                             characters)')
    FAX = models.CharField(blank=True, null=True, max_length=40,
                           help_text='Fax number (maximum length: 40 \
                           characters)')
    CUSTOMEREMAIL = models.CharField(blank=True, null=True, max_length=40,
                                     help_text='Customer\'s e-mail address \
                                     (maximum length: 40 characters)')
    FIRSTNAME_D = models.CharField(blank=True, null=True, max_length=40,
                                   help_text='First name (maximum length: \
                                   40 characters) ')
    LASTAME_D = models.CharField(blank=True, null=True, max_length=40,
                                 help_text='Last Name (maximum length: 40 \
                                 characters)')
    COMPANY_D = models.CharField(blank=True, null=True, max_length=50,
                                 help_text='Company (maximum length: 50 \
                                 characters)')
    ADDRESS1_D = models.CharField(blank=True, null=True, max_length=100,
                                  help_text='Address (maximum length: 100 \
                                  characters)')
    ADDRESS2_D = models.CharField(blank=True, null=True, max_length=100,
                                  help_text='Additional address info \
                                  (maximum length: 100 characters)')
    CITY_D = models.CharField(blank=True, null=True, max_length=30,
                              help_text='City (maximum length: 30 \
                              characters)')
    STATE_D = models.CharField(blank=True, null=True, max_length=30,
                               help_text='State/Sector/County (maximum \
                               length: 30 characters)')
    ZIPCODE_D = models.CharField(blank=True, null=True, max_length=20,
                                 help_text='ZIP/Postal Code (maximum \
                                 length: 20 characters)')
    COUNTRY_D = models.CharField(blank=True, null=True, max_length=50,
                                 help_text='Country (maximum length: 50 \
                                 characters)')
    COUNTRY_D_CODE = models.CharField(blank=True, null=True, max_length=10,
                                      help_text='Country (maximum length: \
                                      10 characters)')
    PHONE_D = models.CharField(blank=True, null=True, max_length=40,
                               help_text='Phone number (maximum length: 40 \
                               characters)')
    EMAIL_D = models.CharField(blank=True, null=True, max_length=40,
                               help_text='E-mail (maximum length: 40 \
                               characters)')
    IPADDRESS = models.CharField(blank=True, null=True, max_length=250,
                                 help_text='Client\'s IP Address (maximum \
                                 length: 250 characters)')
    IPCOUNTRY = models.CharField(blank=True, null=True, max_length=50,
                                 help_text='Client\'s IP Country (maximum \
                                 length: 50 characters)')
    COMPLETE_DATE = models.CharField(blank=True, null=True, max_length=40,
                                     help_text='The order completion date, \
                                     in the following format: Y-m-d H:i:s \
                                     (2012-04-26 15:02:28) .')
    CURRENCY = models.CharField(blank=True, null=True, max_length=10,
                                help_text='The currency in which the order \
                                has been processed. Possible values: RON, \
                                USD, EUR.')
    LANGUAGE = models.CharField(blank=True, null=True, max_length=40,
                                help_text='The language in which the order \
                                has been processed. Possible values: ro, \
                                en, fr, de, it.')

    IPN_PID = models.TextField(blank=True, null=True,
                               help_text='Array with the ID Codes of the \
                               ordered products, in the PayU database (PayU \
                               reference) ')
    IPN_PNAME = models.TextField(blank=True, null=True,
                                 help_text='Array with product names ')
    IPN_PCODE = models.TextField(blank=True, null=True,
                                 help_text='Array with the product codes \
                                 assigned by the vendor in the system (vendor \
                                 reference)')
    IPN_INFO = models.TextField(blank=True, null=True,
                                help_text='Array with additional \
                                information sent for each ordered product (if \
                                they have been sent to PayU)')
    IPN_QTY = models.TextField(blank=True, null=True,
                               help_text='Array with the product quantities')
    IPN_PRICE = models.TextField(blank=True, null=True,
                                 help_text='Array with unit prices per \
                                 product (without VAT), in RON, with \
                                 period/full-stop (.) as decimal place \
                                 separator')
    IPN_VAT = models.TextField(blank=True, null=True,
                               help_text='Array with VAT values per \
                               product, with period "." as decimal place \
                               separator')
    IPN_VER = models.TextField(blank=True, null=True,
                               help_text='Array with product versions \
                               (maximum length: 50 characters)')
    IPN_DISCOUNT = models.TextField(blank=True, null=True,
                                    help_text='Array with the amounts with \
                                    which there has been made a discount \
                                    in a promotion. Including VAT. ')
    IPN_PROMONAME = models.TextField(blank=True, null=True,
                                     help_text='Array with the names of the \
                                     promotions in which the discounts \
                                     specified above have been made.')
    IPN_PROMOCODE = models.TextField(blank=True, null=True,
                                     help_text='Array with the code of the \
                                     promotions in which the discounts \
                                     specified above have been made.')
    IPN_ORDER_COSTS = models.TextField(blank=True, null=True,
                                       help_text='Array with costs for each \
                                       product from order (expressed in \
                                       order\'s currency)')
    IPN_REC_CURRENT_ITERATION_NO = models.TextField(blank=True, null=True,
                                                    help_text='Current recurring \
                                                                  period (avaible \
                                                                  only for recurrent \
                                                                  payments)')
    IPN_REC_ORIGINAL_REFNO = models.TextField(blank=True, null=True,
                                              help_text='Array containing \
                                                            the reference to \
                                                            the original order')
    IPN_REC_INTERVAL = models.TextField(blank=True, null=True,
                                        help_text='Array containing \
                                                      recurring intervals \
                                                      (day/month/week) for \
                                                      each order')
    IPN_REC_EXPIRATION_DATE = models.TextField(blank=True, null=True,
                                               help_text='Array with \
                                                             expiration dates \
                                                             for each \
                                                             recurrence')
    IPN_REC_MULTIPLIER = models.TextField(blank=True, null=True,
                                          help_text='Array with reccurence \
                                                        period (interval x \
                                                        multiplier) for each \
                                                        product from the order')
    IPN_DELIVEREDCODES = models.TextField(blank=True, null=True,
                                          help_text='Array with the codes \
                                                        delivered to the \
                                                        clients, if the PayU \
                                                        contract contains this \
                                                        feature. Each element \
                                                        in the array is \
                                                        represented by a \
                                                        string, having comma \
                                                        (,) as a separator for \
                                                        each sent code, in \
                                                        case the ordered \
                                                        quantity is greater \
                                                        than 1.')
    IPN_DOWNLOAD_LINK = models.CharField(blank=True, null=True, max_length=250,
                                         help_text='Download link of the \
                                                       product delivered to \
                                                       the client')
    IPN_TOTAL = models.TextField(blank=True, null=True,
                                 help_text='Partial total on order line \
                                               (including VAT), with \
                                               period/full-stop (.) as a \
                                               decimal place separator')
    IPN_TOTALGENERAL = models.CharField(blank=True, null=True, max_length=40,
                                        help_text='Total transaction \
                                                      amount, including VAT \
                                                      costs, with \
                                                      period/full-stop (.) as \
                                                      a decimal place \
                                                      separator')
    IPN_SHIPPING = models.CharField(blank=True, null=True, max_length=50,
                                    help_text='Total amount paid for \
                                                  shippment')
    IPN_REFERRER = models.CharField(blank=True, null=True, max_length=250,
                                    help_text='HTTP referrer of the sale.')
    IPN_GLOBALDISCOUNT = models.CharField(blank=True, null=True, max_length=250,
                                          help_text='Global discount of the \
                                                        sale. This field is \
                                                        option and is avaible \
                                                        only if the amount is \
                                                        greater than zero.')
    IPN_COMMISSION = models.CharField(blank=True, null=True, max_length=50,
                                      help_text='Payu\'s commision in RON, \
                                                    with period/full-stop (.) \
                                                    as a decimal place \
                                                    separator.')
    IPN_DATE = models.DateTimeField(blank=True, null=True, max_length=40,
                                    help_text='IPN POST\'s sending date in the \
                                               following format: YmdHMS (ex.: \
                                               20120426145935)')

    response = models.TextField(blank=True)
    ip_address = models.GenericIPAddressField(blank=True, null=True)

    flag = models.BooleanField(default=False)
    flag_info = models.TextField(blank=True)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def set_flag(self, info):
        """
        Sets a flag on the transaction and also sets a reason.
        """

        self.flag = True
        self.flag_info += info

    @property
    def is_authorized(self):
        return self.ORDERSTATUS in ['PAYMENT_AUTHORIZED', 'PAYMENT_RECEIVED', 'TEST']

    @property
    def is_completed(self):
        return self.ORDERSTATUS == 'COMPLETE'

    def __unicode__(self):
        return u'<IPN: %s>' % self.REFNO

    class Meta:
        verbose_name = 'PayU IPN'
        db_table = 'payu_ipn'
        app_label = 'payu'


class PayUIDN(models.Model):
    ipn = models.OneToOneField(PayUIPN, on_delete=models.CASCADE)
    sent = models.BooleanField(default=False)

    success = models.BooleanField(default=False)
    response = models.TextField(blank=True)

    class Meta:
        verbose_name = 'PayU IDN'
        app_label = 'payu'

    def send(self):
        payload = self._build_payload(PAYU_MERCHANT, PAYU_MERCHANT_KEY)

        try:
            response = requests.post(PAYU_IDN_URL, data=payload)

            self.success = response.status_code == 200
            self.response = response.content
        except Exception as e:
            self.response = str(e)
            self.success = False

        self.sent = True
        self.save()

    @classmethod
    def signature(cls, payload, merchant_key):
        hashable_fields = ["MERCHANT", "ORDER_REF", "ORDER_AMOUNT", "ORDER_CURRENCY", "IDN_DATE",
                           "CHARGE_AMOUNT"]

        hash_values = [payload[field] for field in hashable_fields if field in payload]

        confirmation_hash = "".join(
            ['{length}{value}'.format(
                    length=len(str(value).encode('utf-8')), value=value
            ) for value in hash_values]
        ).encode('utf-8')
        return hmac.new(merchant_key, confirmation_hash, hashlib.md5).hexdigest()

    def _build_payload(self, merchant, merchant_key, now=None):
        payload = OrderedDict([
            ('MERCHANT', merchant),
            ('ORDER_REF', self.ipn.REFNO or -1),
            ('ORDER_AMOUNT', self.ipn.IPN_TOTALGENERAL or 0),
            ('ORDER_CURRENCY', self.ipn.CURRENCY or 'RON'),
            ('IDN_DATE', now or datetime.now(pytz.UTC).strftime('%Y-%m-%d %H:%M:%S')),
        ])
        payload["ORDER_HASH"] = self.signature(payload, merchant_key)

        return payload

    def __unicode__(self):
        return u'<IDN: %s>' % self.pk


class PayUToken(models.Model):
    ipn = models.OneToOneField(PayUIPN, on_delete=models.CASCADE)

    # used for token/v1 api payments (same value as IPN's REFNO)
    # DEPRECATED
    IPN_CC_TOKEN = models.CharField(max_length=9, verbose_name="Token (deprecated)", blank=True, null=True)

    # can be used for the `IPN_CC_TOKEN` field in alu/v3 token payments
    TOKEN_HASH = models.CharField(max_length=64, verbose_name="Token", blank=True, null=True)

    # documentation is unclear of the length and format of this field
    IPN_CC_MASK = models.CharField(max_length=36, verbose_name="Last 4 digits")

    IPN_CC_EXP_DATE = models.DateField(verbose_name="Expiration date")

    class Meta:
        verbose_name = 'PayU Tokens'
        app_label = 'payu'

    def __unicode__(self):
        return u'<Token: %s>' % (self.TOKEN_HASH or self.IPN_CC_TOKEN)


@receiver(post_save, sender=PayUToken)
def post_payu_ipn_cc_token_save(sender, instance=None, **kwargs):
    alu_token_created.send(sender=instance)


@receiver(post_save, sender=PayUIPN)
def post_payu_ipn_save(sender, instance=None, **kwargs):
    if instance.flag:
        payment_flagged.send(sender=instance)
        return

    if instance.is_authorized:
        payment_authorized.send(sender=instance)

    if instance.is_completed:
        payment_completed.send(sender=instance)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2020-05-29 10:47
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('payu', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='payutoken',
            name='TOKEN_HASH',
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name=b'Token Hash'),
        ),
    ]



# Generated by Django 3.1.12 on 2021-06-28 12:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('payu', '0003_auto_20200820_0858'),
    ]

    operations = [
        migrations.AlterField(
            model_name='payuipn',
            name='ADDRESS1',
            field=models.CharField(blank=True, help_text='Address (maximum length: 100                                 characters)', max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='ADDRESS1_D',
            field=models.CharField(blank=True, help_text='Address (maximum length: 100                                   characters)', max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='ADDRESS2',
            field=models.CharField(blank=True, help_text='Additional Address info                                 (maximum length: 100 characters)', max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='ADDRESS2_D',
            field=models.CharField(blank=True, help_text='Additional address info                                   (maximum length: 100 characters)', max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='CARD_TYPE',
            field=models.CharField(blank=True, help_text='Used credit card type.                                  Ex: "Visa" or "MasterCard"', max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='CBANKACCOUNT',
            field=models.CharField(blank=True, help_text="Company's Bank Account                                     (maximum length: 50 characters)", max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='CBANKNAME',
            field=models.CharField(blank=True, help_text="Company's Bank (maximum                                  length: 40 characters) ", max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='CITY',
            field=models.CharField(blank=True, help_text='City (maximum length: 30 characters)', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='CITY_D',
            field=models.CharField(blank=True, help_text='City (maximum length: 30                               characters)', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='COMPANY',
            field=models.CharField(blank=True, help_text='Company name (maximum length:                                              40 characters) ', max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='COMPANY_D',
            field=models.CharField(blank=True, help_text='Company (maximum length: 50                                  characters)', max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='COMPLETE_DATE',
            field=models.CharField(blank=True, help_text='The order completion date,                                      in the following format: Y-m-d H:i:s                                      (2012-04-26 15:02:28) .', max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='COUNTRY',
            field=models.CharField(blank=True, help_text='Country (maximum length: 50                                characters)', max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='COUNTRY_CODE',
            field=models.CharField(blank=True, help_text='Country (maximum length: 10                                     characters)', max_length=10, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='COUNTRY_D',
            field=models.CharField(blank=True, help_text='Country (maximum length: 50                                  characters)', max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='COUNTRY_D_CODE',
            field=models.CharField(blank=True, help_text='Country (maximum length:                                       10 characters)', max_length=10, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='CURRENCY',
            field=models.CharField(blank=True, help_text='The currency in which the order                                 has been processed. Possible values: RON,                                 USD, EUR.', max_length=10, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='CUSTOMEREMAIL',
            field=models.CharField(blank=True, help_text="Customer's e-mail address                                      (maximum length: 40 characters)", max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='EMAIL_D',
            field=models.CharField(blank=True, help_text='E-mail (maximum length: 40                                characters)', max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='FAX',
            field=models.CharField(blank=True, help_text='Fax number (maximum length: 40                            characters)', max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='FIRSTNAME',
            field=models.CharField(blank=True, help_text="Client's first name", max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='FIRSTNAME_D',
            field=models.CharField(blank=True, help_text='First name (maximum length:                                    40 characters) ', max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='FISCALCODE',
            field=models.CharField(blank=True, help_text='Unique Registration Number /                                   VAT ID (maximum length: 40 characters)', max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IDENTITY_CNP',
            field=models.CharField(blank=True, help_text="Customer's personal numeric                                     code, available only for Romanian customers.", max_length=13, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IDENTITY_ISSUER',
            field=models.CharField(blank=True, help_text='IDENTITY_NO ID Card                                        issuer authority ', max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IDENTITY_NO',
            field=models.CharField(blank=True, help_text='Customer ID Card series and                                    number (Series / Number - available                                    only for Romanian customers)', max_length=15, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPADDRESS',
            field=models.CharField(blank=True, help_text="Client's IP Address (maximum                                  length: 250 characters)", max_length=250, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPCOUNTRY',
            field=models.CharField(blank=True, help_text="Client's IP Country (maximum                                  length: 50 characters)", max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_COMMISSION',
            field=models.CharField(blank=True, help_text="Payu's commision in RON,                                                     with period/full-stop (.)                                                     as a decimal place                                                     separator.", max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_DATE',
            field=models.DateTimeField(blank=True, help_text="IPN POST's sending date in the                                                following format: YmdHMS (ex.:                                                20120426145935)", max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_DELIVEREDCODES',
            field=models.TextField(blank=True, help_text='Array with the codes                                                         delivered to the                                                         clients, if the PayU                                                         contract contains this                                                         feature. Each element                                                         in the array is                                                         represented by a                                                         string, having comma                                                         (,) as a separator for                                                         each sent code, in                                                         case the ordered                                                         quantity is greater                                                         than 1.', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_DISCOUNT',
            field=models.TextField(blank=True, help_text='Array with the amounts with                                     which there has been made a discount                                     in a promotion. Including VAT. ', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_DOWNLOAD_LINK',
            field=models.CharField(blank=True, help_text='Download link of the                                                        product delivered to                                                        the client', max_length=250, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_GLOBALDISCOUNT',
            field=models.CharField(blank=True, help_text='Global discount of the                                                         sale. This field is                                                         option and is avaible                                                         only if the amount is                                                         greater than zero.', max_length=250, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_INFO',
            field=models.TextField(blank=True, help_text='Array with additional                                 information sent for each ordered product (if                                 they have been sent to PayU)', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_ORDER_COSTS',
            field=models.TextField(blank=True, help_text="Array with costs for each                                        product from order (expressed in                                        order's currency)", null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_PCODE',
            field=models.TextField(blank=True, help_text='Array with the product codes                                  assigned by the vendor in the system (vendor                                  reference)', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_PID',
            field=models.TextField(blank=True, help_text='Array with the ID Codes of the                                ordered products, in the PayU database (PayU                                reference) ', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_PNAME',
            field=models.TextField(blank=True, help_text='Array with product names ', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_PRICE',
            field=models.TextField(blank=True, help_text='Array with unit prices per                                  product (without VAT), in RON, with                                  period/full-stop (.) as decimal place                                  separator', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_PROMOCODE',
            field=models.TextField(blank=True, help_text='Array with the code of the                                      promotions in which the discounts                                      specified above have been made.', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_PROMONAME',
            field=models.TextField(blank=True, help_text='Array with the names of the                                      promotions in which the discounts                                      specified above have been made.', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_QTY',
            field=models.TextField(blank=True, help_text='Array with the product quantities', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_REC_CURRENT_ITERATION_NO',
            field=models.TextField(blank=True, help_text='Current recurring                                                                   period (avaible                                                                   only for recurrent                                                                   payments)', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_REC_EXPIRATION_DATE',
            field=models.TextField(blank=True, help_text='Array with                                                              expiration dates                                                              for each                                                              recurrence', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_REC_INTERVAL',
            field=models.TextField(blank=True, help_text='Array containing                                                       recurring intervals                                                       (day/month/week) for                                                       each order', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_REC_MULTIPLIER',
            field=models.TextField(blank=True, help_text='Array with reccurence                                                         period (interval x                                                         multiplier) for each                                                         product from the order', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_REC_ORIGINAL_REFNO',
            field=models.TextField(blank=True, help_text='Array containing                                                             the reference to                                                             the original order', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_REFERRER',
            field=models.CharField(blank=True, help_text='HTTP referrer of the sale.', max_length=250, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_SHIPPING',
            field=models.CharField(blank=True, help_text='Total amount paid for                                                   shippment', max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_TOTAL',
            field=models.TextField(blank=True, help_text='Partial total on order line                                                (including VAT), with                                                period/full-stop (.) as a                                                decimal place separator', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_TOTALGENERAL',
            field=models.CharField(blank=True, help_text='Total transaction                                                       amount, including VAT                                                       costs, with                                                       period/full-stop (.) as                                                       a decimal place                                                       separator', max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_VAT',
            field=models.TextField(blank=True, help_text='Array with VAT values per                                product, with period "." as decimal place                                separator', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='IPN_VER',
            field=models.TextField(blank=True, help_text='Array with product versions                                (maximum length: 50 characters)', null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='LANGUAGE',
            field=models.CharField(blank=True, help_text='The language in which the order                                 has been processed. Possible values: ro,                                 en, fr, de, it.', max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='LASTAME_D',
            field=models.CharField(blank=True, help_text='Last Name (maximum length: 40                                  characters)', max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='LASTNAME',
            field=models.CharField(blank=True, help_text="Client's last name", max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='ORDERNO',
            field=models.CharField(max_length=6, verbose_name='Merchant order #'),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='ORDERSTATUS',
            field=models.CharField(choices=[('PAYMENT_AUTHORIZED', 'PAYMENT_AUTHORIZED'), ('PAYMENT_RECEIVED', 'PAYMENT_RECEIVED'), ('TEST', 'TEST'), ('CASH', 'CASH'), ('COMPLETE', 'COMPLETE'), ('REVERSED', 'REVERSED'), ('REFUND', 'REFUND')], max_length=18, verbose_name='Status'),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='PAYMENTDATE',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Payment date'),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='PAYMETHOD',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Payment method'),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='PAYMETHOD_CODE',
            field=models.CharField(max_length=10, verbose_name='Payment method code'),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='PHONE',
            field=models.CharField(blank=True, help_text='Phone number (maximum length: 40                              characters)', max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='PHONE_D',
            field=models.CharField(blank=True, help_text='Phone number (maximum length: 40                                characters)', max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='REFNO',
            field=models.CharField(max_length=9, verbose_name='ePayment reference'),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='REFNOEXT',
            field=models.CharField(max_length=100, verbose_name='Merchant reference'),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='REGISTRATIONNUMBER',
            field=models.CharField(blank=True, help_text="Company's Commerce                                           Registry registration number                                           (maximum length: 40 characters)", max_length=40, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='SALEDATE',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Sale date'),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='STATE',
            field=models.CharField(blank=True, help_text='State/Sector/County (maximum                              length: 30 characters)', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='STATE_D',
            field=models.CharField(blank=True, help_text='State/Sector/County (maximum                                length: 30 characters)', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='ZIPCODE',
            field=models.CharField(blank=True, help_text='ZIP/Postal Code (maximum length:                                20 characters)', max_length=20, null=True),
        ),
        migrations.AlterField(
            model_name='payuipn',
            name='ZIPCODE_D',
            field=models.CharField(blank=True, help_text='ZIP/Postal Code (maximum                                  length: 20 characters)', max_length=20, null=True),
        ),
        migrations.AlterField(
            model_name='payutoken',
            name='IPN_CC_EXP_DATE',
            field=models.DateField(verbose_name='Expiration date'),
        ),
        migrations.AlterField(
            model_name='payutoken',
            name='IPN_CC_MASK',
            field=models.CharField(max_length=36, verbose_name='Last 4 digits'),
        ),
        migrations.AlterField(
            model_name='payutoken',
            name='IPN_CC_TOKEN',
            field=models.CharField(blank=True, max_length=9, null=True, verbose_name='Token (deprecated)'),
        ),
        migrations.AlterField(
            model_name='payutoken',
            name='TOKEN_HASH',
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name='Token'),
        ),
    ]



# Copyright (c) 2017 Presslabs SRL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime

import pytest
import django
from django.http import QueryDict

from payu.forms import (PayULiveUpdateForm, ValueHiddenInput, OrderWidget,
                        PayUIPNForm)


@pytest.mark.parametrize("payload,signature", [
    ({
        'ORDER_REF': 112457,
        'ORDER_DATE': '2012-05-01 15:51:35',
        'ORDER': [
            {
                'PNAME': 'MacBook Air 13 inch',
                'PCODE': 'MBA13',
                'PINFO': 'Extended Warranty - 5 Years',
                'PRICE': 1750, 'PRICE_TYPE': 'GROSS',
                'QTY': 1,
                'VAT': 24
            },
        ],
        'BILL_FNAME': 'Joe',
        'BILL_LNAME': 'Doe',
        'BILL_COUNTRYCODE': 'RO',
        'BILL_PHONE': '+040000000000',
        'BILL_EMAIL': 'joe.doe@gmail.com',
        'BILL_COMPANY': 'ACME Inc',
        'BILL_FISCALCODE': None,
        'PRICES_CURRENCY': 'RON',
        'CURRENCY': 'RON',
        'PAY_METHOD': 'CCVISAMC'
    }, '5b118e083e52a24872f579134c5db6cc'),
    ({
        'ORDER_REF': 112457,
        'ORDER_DATE': '2012-05-01 15:51:35',
        'ORDER': [
            {
                'PNAME': 'MacBook Air 13 inch',
                'PCODE': 'MBA13',
                'PINFO': '',
                'PRICE': 1750,
                'PRICE_TYPE': 'GROSS',
                'QTY': 1,
                'VAT': 24
            },
        ],
        'BILL_FNAME': 'Joe',
        'BILL_LNAME': 'Doe',
        'BILL_COUNTRYCODE': 'RO',
        'BILL_PHONE': '+040000000000',
        'BILL_EMAIL': 'joe.doe@gmail.com',
        'BILL_COMPANY': 'ACME Inc',
        'BILL_FISCALCODE': None,
        'PRICES_CURRENCY': 'RON',
        'CURRENCY': 'RON',
        'LANGUAGE': 'RO',
        'TESTORDER': 'TRUE',
        'PAY_METHOD': 'CCVISAMC'
    }, 'e327589caadf200996521a4d0b433ef5'),
    ({
        'ORDER_REF': '789456123',
        'ORDER_DATE': '2016-10-05 11:12:27',
        'ORDER': [
            {
                'PNAME': 'CD Player',
                'PCODE': 'PROD_04891',
                'PINFO': 'Extended Warranty - 5 Years',
                'PRICE': '82.3',
                'PRICE_TYPE': 'GROSS',
                'QTY': '7',
                'VAT':'20'
            },
            {
                'PNAME': 'Mobile Phone',
                'PCODE': 'PROD_07409',
                'PINFO': 'Dual SIM',
                'PRICE': '1945.75',
                'PRICE_TYPE': 'GROSS',
                'QTY': '3',
                'VAT':'20'
            },
            {
                'PNAME': 'Laptop',
                'PCODE': 'PROD_04965',
                'PINFO': '17" Display',
                'PRICE': '5230',
                'PRICE_TYPE': 'GROSS',
                'QTY': '1',
                'VAT':'20'
            },
        ],
        'PRICES_CURRENCY': 'RON',
        'ORDER_SHIPPING': '0',
        'DISCOUNT': '55',
        'PAY_METHOD': 'CCVISAMC',
        'DESTINATION_CITY': 'Bucuresti',
        'DESTINATION_STATE': 'Bucuresti',
        'DESTINATION_COUNTRY': 'RO',
        'TESTORDER': 'TRUE',
        'BILL_FNAME': 'Joe',
        'BILL_LNAME': 'Doe',
        'BILL_COUNTRYCODE': 'RO',
        'BILL_PHONE': '+040000000000',
        'BILL_EMAIL': 'joe.doe@gmail.com',
    }, 'a9b838e17a04cc045699e5501f8f12c6')
])
def test_calculate_correct_hash(payload, signature):
    payu_form = PayULiveUpdateForm(initial=payload)
    assert payu_form.signature == signature
    assert payu_form.fields['ORDER_HASH'].initial == signature


@pytest.mark.parametrize("payload,orders", [
    ({
        'ORDER': [
            {
                'PNAME': 'MacBook Air 13 inch',
                'PCODE': 'MBA13',
                'PINFO': 'Extended Warranty - 5 Years',
                'PRICE': 1750,
                'PRICE_TYPE': 'GROSS',
                'QTY': 1,
                'VAT': 24
            },
        ],
    }, [{
        'VER': None,
        'PRICE_TYPE': 'GROSS',
        'PRICE': 1750,
        'QTY': 1,
        'PINFO': 'Extended Warranty - 5 Years',
        'PCODE': 'MBA13',
        'PNAME': 'MacBook Air 13 inch',
        'PGROUP': None,
        'VAT': 24
    }]),
    ({
        'ORDER': [
            {
                'PNAME': 'MacBook Air 13 inch',
                'PCODE': 'MBA13',
                'PRICE': 1750,
                'PRICE_TYPE': 'GROSS',
            },
        ],
    }, [{
        'VER': None,
        'PRICE_TYPE': 'GROSS',
        'PRICE': 1750,
        'QTY': 1,
        'PINFO': None,
        'PCODE': 'MBA13',
        'PNAME': 'MacBook Air 13 inch',
        'PGROUP': None,
        'VAT': 24
    }])
])
def test_orders_parsing(payload, orders):
    payu_form = PayULiveUpdateForm(initial=payload)
    assert payu_form._prepare_orders(payload['ORDER']) == orders


@pytest.mark.parametrize("field,html", [
    (ValueHiddenInput().render('name', None), ''),

    (ValueHiddenInput().render('name', ''), ''),
    (ValueHiddenInput().render('name', 'value'),
     '<input type="hidden" name="name" value="value" />'),
    (ValueHiddenInput().render('ORDER_10_0', 'a'),
     '<input type="hidden" name="ORDER_PNAME[]" value="a" />'),
    (ValueHiddenInput().render('ORDER_10_10', 'a'),
     '<input type="hidden" name="ORDER_10_10" value="a" />'),
])
def test_value_input_hidden(field, html):
    assert field == html


@pytest.mark.parametrize("decompressed_value,expected_value", [
    (OrderWidget().decompress({'n': 'test', 'm': 20}),
     ['', '', '', '', '', '', '', '', '']),
    (OrderWidget().decompress({'PNAME': 'test', 'VAT': 20}),
     ['test', '', '', '', '', '', '', 20, '']),
    (OrderWidget().decompress({'PNAME': '1', 'VAT': 20, 'PGROUP': '2',
                               'PCODE': '3', 'PINFO': '4', 'PRICE': '5'}),
     ['1', '2', '3', '4', '5', '', '', 20, ''])
])
def test_decompress_order_widget(decompressed_value, expected_value):
    assert decompressed_value == expected_value


@pytest.mark.parametrize("form_data,expected", [
    ({}, lambda form: not form.is_valid()),
    ({'REFNO': 1}, lambda form: not form.is_valid()),
    ({'REFNO': 1, 'REFNOEXT': 1}, lambda form: not form.is_valid()),
    ({'REFNO': 1, 'REFNOEXT': 1, 'ORDERNO': 1, 'ORDERSTATUS': 'TEST',
      'HASH': 'aaa', 'PAYMETHOD_CODE': 'CCVMAC'}, lambda form: form.is_valid()),
])
def test_payu_model_form_validation(form_data, expected):
    assert expected(PayUIPNForm(form_data))


def test_payu_model_form_date_conversion():
    form = PayUIPNForm({'IPN_DATE': '20161220170852'})
    assert form['IPN_DATE'].value() == datetime.strptime('20161220170852',
                                                         '%Y%m%d%H%M%S')


@pytest.mark.parametrize("form_data,expected", [
    (QueryDict('IPN_PID[]=1&IPN_PID[]=2&IPN_PID[]=3'), '1,2,3'),
    (QueryDict('IPN_PID=1'), '1'),
    (QueryDict('IPN_PID[]=1'), '1'),
    (QueryDict('IPN_PID[]='), ''),
    (QueryDict('IPN_PID[=1'), None)
])
def test_payu_model_form_list_processing(form_data, expected):
    form = PayUIPNForm(form_data)
    assert form['IPN_PID'].value() == expected


@pytest.mark.parametrize("payload, html", [
    ({
        'ORDER_REF': 112457,
        'ORDER_DATE': '2012-05-01 15:51:35',
        'ORDER': [
            {
                'PNAME': 'MacBook Air 13 inch',
                'PCODE': 'MBA13',
                'PINFO': 'Extended Warranty - 5 Years',
                'PRICE': 1750,
                'PRICE_TYPE': 'GROSS',
                'QTY': 1,
                'VAT': 24
            },
        ],
        'BILL_FNAME': 'Joe',
        'BILL_LNAME': 'Doe',
        'BILL_COUNTRYCODE': 'RO',
        'BILL_PHONE': '+040000000000',
        'BILL_EMAIL': 'joe.doe@gmail.com',
        'BILL_COMPANY': 'ACME Inc',
        'BILL_FISCALCODE': None,
        'PRICES_CURRENCY': 'RON',
        'CURRENCY': 'RON',
        'PAY_METHOD': 'CCVISAMC'
     }, '''
<input type="hidden" name="MERCHANT" value="PAYUDEMO" id="id_MERCHANT" /><input type="hidden" name="ORDER_REF" value="112457" id="id_ORDER_REF" /><input type="hidden" name="ORDER_DATE" value="2012-05-01 15:51:35" id="id_ORDER_DATE" /><input type="hidden" name="ORDER_PNAME[]" value="MacBook Air 13 inch" id="id_ORDER_0_0" /><input type="hidden" name="ORDER_PCODE[]" value="MBA13" id="id_ORDER_0_2" /><input type="hidden" name="ORDER_PINFO[]" value="Extended Warranty - 5 Years" id="id_ORDER_0_3" /><input type="hidden" name="ORDER_PRICE[]" value="1750" id="id_ORDER_0_4" /><input type="hidden" name="ORDER_PRICE_TYPE[]" value="GROSS" id="id_ORDER_0_5" /><input type="hidden" name="ORDER_QTY[]" value="1" id="id_ORDER_0_6" /><input type="hidden" name="ORDER_VAT[]" value="24" id="id_ORDER_0_7" /><input type="hidden" name="PRICES_CURRENCY" value="RON" id="id_PRICES_CURRENCY" /><input type="hidden" name="PAY_METHOD" value="CCVISAMC" id="id_PAY_METHOD" /><input type="hidden" name="ORDER_HASH" value="5b118e083e52a24872f579134c5db6cc" id="id_ORDER_HASH" /><input type="hidden" name="BILL_FNAME" value="Joe" id="id_BILL_FNAME" /><input type="hidden" name="BILL_LNAME" value="Doe" id="id_BILL_LNAME" /><input type="hidden" name="BILL_COUNTRYCODE" value="RO" id="id_BILL_COUNTRYCODE" /><input type="hidden" name="BILL_PHONE" value="+040000000000" id="id_BILL_PHONE" /><input type="hidden" name="BILL_EMAIL" value="joe.doe@gmail.com" id="id_BILL_EMAIL" /><input type="hidden" name="BILL_COMPANY" value="ACME Inc" id="id_BILL_COMPANY" /><input type="hidden" name="CURRENCY" value="RON" id="id_CURRENCY" /><input type="hidden" name="AUTOMODE" value="1" id="id_AUTOMODE" /><input type="hidden" name="LANGUAGE" value="EN" id="id_LANGUAGE" /><input type="hidden" name="TESTORDER" value="TRUE" id="id_TESTORDER" />
     ''')
])
def test_html_rendering(payload, html):
    payu_form = PayULiveUpdateForm(initial=payload)
    assert "".join(payu_form.as_p().split("\n")) == html.split("\n")[1]



# Copyright (c) 2017 Presslabs SRL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest
from mock import patch
from django_dynamic_fixture import G

from django.core.management import call_command

from payu.models import PayUIDN


@pytest.mark.django_db
@patch('payu.models.requests')
def test_send_idn_payu(mocked_requests):
    mocked_requests.post().status_code = 200
    mocked_requests.post().content = "ok"

    idns = [G(PayUIDN) for _ in range(4)]

    idns_args = [str(idn.pk) for idn in idns[:2]]
    call_command('send_idns', '--idns=%s' % ','.join(idns_args))

    for idn in idns[:2]:
        idn.refresh_from_db()

        assert idn.sent
        assert idn.success
        assert idn.response == "ok"


@pytest.mark.django_db
@patch('payu.models.requests')
def test_send_idn_payu_fail(mocked_requests):
    mocked_requests.post.side_effect = Exception("error")

    idn = G(PayUIDN)

    call_command('send_idns')

    idn.refresh_from_db()

    assert idn.sent
    assert not idn.success
    assert idn.response == "error"



# -*- encoding: utf-8 -*-
# Copyright 2012-2016 PressLabs SRL
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""
Python setup file for the django-silver app.

In order to register your app at pypi.python.org, create an account at
pypi.python.org and login, then register your new app like so:

    python setup.py register

If your name is still free, you can now make your first release but first you
should check if you are uploading the correct files:

    python setup.py sdist

Inspect the output thoroughly. There shouldn't be any temp files and if your
app includes staticfiles or templates, make sure that they appear in the list.
If something is wrong, you need to edit MANIFEST.in and run the command again.
If all looks good, you can make your first release:

    python setup.py sdist upload

For new releases, you need to bump the version number in
silver/__init__.py and re-run the above command.

For more information on creating source distributions, see
http://docs.python.org/2/distutils/sourcedist.html
"""

from setuptools import setup, find_packages

import payu

install_requires = [line.strip()
                    for line in open("requirements.txt").readlines()
                    if not line.strip().startswith('#') and line.strip()]

setup(
    name='django-payu-ro',
    version=payu.__version__,
    author='Presslabs SRL',
    author_email='support@presslabs.com',
    url='http://github.com/PressLabs/django-payu-ro',
    description='A pluggable Django application for integrating PayU Payments (ex. ePayment)',
    packages=find_packages(),
    include_package_data=True,
    install_requires=install_requires,
    classifiers=[
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Operating System :: OS Independent",
        "Topic :: Software Development"
    ],
)



# coding=utf-8
#
# Copyright 2012-2016 PressLabs SRL
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
import hashlib
import re
import hmac
from datetime import datetime

from django import forms

from payu.models import PayUIPN
from payu.conf import (PAYU_MERCHANT, PAYU_MERCHANT_KEY, PAYU_TEST_TRANSACTION,
                       PAYU_ORDER_DETAILS, PAYU_ORDER_DETAILS_DEFAULTS,
                       PAYU_DATE_FORMATS, PAYU_CURRENCIES,
                       PAYU_PAYMENT_METHODS, PAYU_LANGUAGES,
                       PAYU_LU_CALLBACK)


class ValueHiddenInput(forms.HiddenInput):
    """
    Widget that renders only if it has a value.
    Used to remove unused fields from PayU buttons.
    """

    template_name = 'custom_hidden.html'

    def _get_name(self, name):
        detail = re.match(r'^ORDER_(\d+)_(\d+)$', name)
        if detail and int(detail.group(2)) < len(PAYU_ORDER_DETAILS):
            name = 'ORDER_%s[]' % PAYU_ORDER_DETAILS[int(detail.group(2))]
        return name

    def get_context(self, name, value, attrs):
        context = super(ValueHiddenInput, self).get_context(name, value, attrs)
        context['widget']['name'] = self._get_name(context['widget']['name'])
        return context

    def render(self, name, value, *args, **kwargs):
        if value is None:
            return ""

        name = self._get_name(name)

        return super(ValueHiddenInput, self).render(name, value or "", *args, **kwargs)


class OrderWidget(forms.MultiWidget):
    def __init__(self, attrs=None, *args, **kwargs):
        attrs = attrs or {}
        all_widgets = [ValueHiddenInput(attrs) for _ in PAYU_ORDER_DETAILS]
        super(OrderWidget, self).__init__(all_widgets, *args, **kwargs)

    def decompress(self, value):
        return [value.get(detail, '') for detail in PAYU_ORDER_DETAILS]


class OrderField(forms.MultiValueField):
    widget = OrderWidget

    def __init__(self, *args, **kwargs):
        all_fields = tuple(forms.CharField() for _ in PAYU_ORDER_DETAILS)
        super(OrderField, self).__init__(all_fields, *args, **kwargs)


class OrdersWidget(forms.MultiWidget):
    is_hidden = True

    def __init__(self, count, *args, **kwargs):
        all_widgets = tuple((OrderWidget()) for _ in range(count))
        super(OrdersWidget, self).__init__(all_widgets, *args, **kwargs)


class OrdersField(forms.MultiValueField):
    def __init__(self, *args, **kwargs):
        products = kwargs.get('initial', [])
        kwargs['label'] = ''

        all_fields = tuple()
        if products:
            self.widget = OrdersWidget(len(products))
            all_fields = tuple((OrderField()) for _ in products)
        super(OrdersField, self).__init__(all_fields, *args, **kwargs)


class PayULiveUpdateForm(forms.Form):
    MERCHANT = forms.CharField(widget=ValueHiddenInput,
                               initial=PAYU_MERCHANT)
    LU_ENABLE_TOKEN = forms.CharField(widget=ValueHiddenInput, initial='')
    LU_TOKEN_TYPE = forms.CharField(widget=ValueHiddenInput, initial='')
    ORDER_REF = forms.CharField(widget=ValueHiddenInput, initial='')
    ORDER_DATE = forms.CharField(widget=ValueHiddenInput,
                                 initial=datetime.now().strftime('%Y-%m-%d %H:%M:%S'))

    ORDER = OrdersField()
    ORDER_SHIPPING = forms.CharField(widget=ValueHiddenInput)
    PRICES_CURRENCY = forms.ChoiceField(widget=ValueHiddenInput,
                                        choices=PAYU_CURRENCIES, initial='USD')
    DISCOUNT = forms.CharField(widget=ValueHiddenInput)

    DESTINATION_CITY = forms.CharField(widget=ValueHiddenInput)
    DESTINATION_STATE = forms.CharField(widget=ValueHiddenInput)
    DESTINATION_COUNTRY = forms.CharField(widget=ValueHiddenInput)

    PAY_METHOD = forms.ChoiceField(widget=ValueHiddenInput,
                                   choices=PAYU_PAYMENT_METHODS)

    ORDER_HASH = forms.CharField(widget=ValueHiddenInput,
                                 initial='')

    BILL_FNAME = forms.CharField(widget=ValueHiddenInput)
    BILL_LNAME = forms.CharField(widget=ValueHiddenInput)
    BILL_COUNTRYCODE = forms.CharField(widget=ValueHiddenInput)
    BILL_CITY = forms.CharField(widget=ValueHiddenInput)
    BILL_PHONE = forms.CharField(widget=ValueHiddenInput)
    BILL_EMAIL = forms.CharField(widget=ValueHiddenInput)
    BILL_COMPANY = forms.CharField(widget=ValueHiddenInput)
    BILL_FISCALCODE = forms.CharField(widget=ValueHiddenInput)

    CURRENCY = forms.ChoiceField(widget=ValueHiddenInput,
                                 choices=PAYU_CURRENCIES, initial='USD')
    AUTOMODE = forms.CharField(widget=ValueHiddenInput,
                               initial='1')
    LANGUAGE = forms.ChoiceField(widget=ValueHiddenInput,
                                 choices=PAYU_LANGUAGES, initial='EN')
    SELECTED_INSTALLMENTS_NO = forms.CharField(widget=ValueHiddenInput)
    BACK_REF = forms.CharField(widget=ValueHiddenInput,
                               initial=PAYU_LU_CALLBACK)
    TESTORDER = forms.CharField(widget=ValueHiddenInput,
                                initial=str(PAYU_TEST_TRANSACTION).upper())

    @property
    def signature(self):
        """
        Compute the ORDER_HASH of the request.

        The hashable string is composed by getting the values from:
            MERCHANT
            ORDER_REF
            ORDER_DATE
            ORDER_PNAME[]
            ORDER_PCODE[]
            ORDER_PINFO[]
            ORDER_PRICE[]
            ORDER_QTY[]
            ORDER_VAT[]
            ORDER_SHIPPING
            PRICES_CURRENCY
            DISCOUNT
            DESTINATION_CITY
            DESTINATION_STATE
            DESTINATION_COUNTRY
            PAY_METHOD
            ORDER_PRICE_TYPE[]
            SELECTED_INSTALLMENTS_NO
            TESTORDER
        in this exact order. Next, we need to concatenate their lenghts with
        thier values, resulting in a string like:

        8PAYUDEMO9789456123192016-10-05 11:12:279CD Player12MobilePhone6Laptop
        10PROD_0489110PROD_0740910PROD_0496527Extended Warranty - 5 Years8
        Dual SIM1117"Display482.371945.7545230171311220220220103RON2559
        Bucuresti9Bucuresti2RO8CCVISAMC5GROSS5GROSS5GROSS4TRUE

        Using this string and the MERCHANT_KEY, we compute the HMAC.
        """

        hashable_fields = ['MERCHANT', 'ORDER_REF', 'ORDER_DATE',
                           'ORDER_SHIPPING', 'PRICES_CURRENCY', 'DISCOUNT',
                           'DESTINATION_CITY', 'DESTINATION_STATE',
                           'DESTINATION_COUNTRY', 'PAY_METHOD',
                           'SELECTED_INSTALLMENTS_NO', 'TESTORDER']
        result = ""

        # We need this hack since payU is not consistent
        # with the order of fields in hash string

        suffix = ""
        for field in self:
            if field.name == 'ORDER_HASH':
                continue

            field_value = field.value()

            if field.name in hashable_fields and field_value:
                encoded_value = '{length}{value}'.format(
                    length=len(str(field_value).encode('utf-8')), value=field_value
                )
                if field.name == 'TESTORDER' or \
                    field.name == 'SELECTED_INSTALLMENTS_NO':
                    suffix += encoded_value
                else:
                    result += encoded_value

            if field.name == 'ORDER':
                for detail in PAYU_ORDER_DETAILS:
                    if any([detail in order and order[detail]
                            for order in field_value]):

                        for order in field_value:
                            value = order.get(detail, '')

                            item = '{length}{value}'.format(
                                length=len(str(value).encode('utf-8')), value=value
                            )

                            if detail == 'PRICE_TYPE':
                                suffix += item
                            else:
                                result += item

        result += suffix
        result = result.encode('utf-8')
        return hmac.new(PAYU_MERCHANT_KEY, result, hashlib.md5).hexdigest()

    def _prepare_orders(self, orders):
        """
        Each order needs to have all it's details filled with default value,
        or None, in case those are not already filled.
        """

        for detail in PAYU_ORDER_DETAILS:
            if not any([detail in order for order in orders]):
                for order in orders:
                    order[detail] = PAYU_ORDER_DETAILS_DEFAULTS.get(detail, None)

        return orders

    def __init__(self, **kwargs):
        initial = kwargs.get('initial', {})
        orders = self._prepare_orders(initial.get('ORDER', []))

        super(PayULiveUpdateForm, self).__init__(**kwargs)

        self.fields['ORDER'] = OrdersField(initial=orders)
        self.fields['ORDER_HASH'].initial = self.signature


class PayUIPNForm(forms.ModelForm):
    class Meta:
        exclude = []
        model = PayUIPN

    def __init__(self, data, *args, **kwargs):
        form_data = data.copy()

        for field in data:
            if field.endswith("[]"):
                form_data[field[:-2]] = ",".join([value
                                                  for value in data.getlist(field)
                                                  if value.strip()
                                                 ])

            if field == 'IPN_DATE':
                form_data[field] = datetime.strptime(data[field], "%Y%m%d%H%M%S")

        super(PayUIPNForm, self).__init__(form_data)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-08-20 08:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('payu', '0002_payutoken_token_hash'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='payutoken',
            options={'verbose_name': 'PayU Tokens'},
        ),
        migrations.AlterField(
            model_name='payutoken',
            name='IPN_CC_TOKEN',
            field=models.CharField(blank=True, max_length=9, null=True, verbose_name=b'Token (deprecated)'),
        ),
        migrations.AlterField(
            model_name='payutoken',
            name='TOKEN_HASH',
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name=b'Token'),
        ),
    ]



# Copyright (c) 2017 Presslabs SRL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest
from mock import patch, MagicMock
from django_dynamic_fixture import G

from payu.models import PayUIPN, PayUIDN
from payu.conf import PAYU_IDN_URL


@pytest.mark.parametrize('flag_info, extra_info', [
    ('', ''),
    ('1', ''),
    ('1', '1'),
])
def test_payu_model_flag(flag_info, extra_info):
    model = PayUIPN(flag_info=flag_info)
    model.set_flag(extra_info)
    assert model.flag_info == flag_info + extra_info
    assert model.flag


@pytest.mark.parametrize('order, authorized, completed', [
    ('1', False, False),
    ('TEST', True, False),
    ('PAYMENT_RECEIVED', True, False),
    ('PAYMENT_AUTHORIZED', True, False),
    ('COMPLETE', False, True),
    ('', False, False)
])
def test_payu_model_order_status(order, authorized, completed):
    model = PayUIPN(ORDERSTATUS=order)
    assert model.is_authorized == authorized
    assert model.is_completed == completed


@pytest.mark.django_db
@patch('payu.models.payment_authorized')
def test_payu_model_authorized_signals(mock_authorized):
    model = G(PayUIPN, ORDERSTATUS='TEST')
    mock_authorized.send.assert_called_once_with(sender=model)


@pytest.mark.django_db
@patch('payu.models.payment_completed')
def test_payu_model_completed_signals(mock_completed):
    model = G(PayUIPN, ORDERSTATUS='COMPLETE')
    mock_completed.send.assert_called_once_with(sender=model)


@pytest.mark.django_db
@patch('payu.models.payment_flagged')
def test_payu_model_flagged_signals(mock_flagged):
    model = G(PayUIPN, flag=True)
    mock_flagged.send.assert_called_once_with(sender=model)


@pytest.mark.parametrize('payload, merchant_key, signature', [
    (
        {}, b'0', '477f91ddbcc6839b9950045977da3530'
    ), (
        # example from payu docs (with actually wrong hash)
        # check online: source b"8MERCHANT73954142539.993USD192015-05-11 14:32:08"
        #               key b"1231234567890123"
        {'MERCHANT': 'MERCHANT', 'ORDER_REF': 3954142, 'ORDER_AMOUNT': 39.99,
         'ORDER_CURRENCY': 'USD', 'IDN_DATE': '2015-05-11 14:32:08'},
        b'1231234567890123', 'e40a3d1505cbb15e34cf35f34c784bb0'
)])
def test_idn_signature(payload, merchant_key, signature):
    assert PayUIDN.signature(payload, merchant_key) == signature


@pytest.mark.parametrize('payu_idn, merchant, merchant_key, now, expected_payload', [
    (PayUIPN(REFNO=1, IPN_TOTALGENERAL=1, CURRENCY='USD'), '123', b'1', 1,
     {
         'MERCHANT': '123',
         'ORDER_REF': 1,
         'ORDER_AMOUNT': 1,
         'ORDER_CURRENCY': 'USD',
         'IDN_DATE': 1,
         'ORDER_HASH': '5d275f182089bab5a5351613067b95dc'
     }),
    (PayUIPN(), '123', b'1', 1,
     {
         'MERCHANT': '123',
         'ORDER_REF': -1,
         'ORDER_AMOUNT': 0,
         'ORDER_CURRENCY': 'RON',
         'IDN_DATE': 1,
         'ORDER_HASH': '61ff18cdfe071679176f03664b4feb72'
     })

])
def test_idn_payload(payu_idn, merchant, merchant_key, now, expected_payload):
    idn = PayUIDN(ipn=payu_idn)
    assert dict(**idn._build_payload(merchant, merchant_key, now)) == expected_payload


@pytest.mark.django_db
@patch('payu.models.requests')
def test_payu_idn_send_success(mocked_requests):
    mocked_requests.post.return_value = MagicMock(status_code=200, content="ok")

    idn = PayUIDN(ipn=G(PayUIPN))
    idn._build_payload = MagicMock(return_value="payload")

    idn.send()

    mocked_requests.post.assert_called_once_with(PAYU_IDN_URL, data="payload")
    assert idn.sent
    assert idn.success
    assert idn.response == "ok"


@pytest.mark.django_db
@patch('payu.models.requests')
def test_payu_idn_send_fail(mocked_requests):
    mocked_requests.post.side_effect = Exception('error')

    idn = PayUIDN(ipn=G(PayUIPN))
    idn._build_payload = MagicMock(return_value="payload")

    idn.send()

    mocked_requests.post.assert_called_once_with(PAYU_IDN_URL, data="payload")
    assert idn.sent
    assert not idn.success
    assert idn.response == "error"
